// Type definitions for websql
// Project: http://www.w3.org/TR/webdatabase/
// Definitions by: TeamworkGuy2 <https://github.com/TeamworkGuy2>
// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped

// W3C spec: http://www.w3.org/TR/webdatabase/#database
// Spec revision: 2010-11-18
// NOTE: the W3C websql spec has been deprecated

// uncomment to integrate with Window global object
interface Window extends WindowDatabase { }
interface WorkerUtils extends WorkerUtilsDatabase { }

//[Supplemental, NoInterfaceObject]
interface WindowDatabase {
    openDatabase(name: string, version: string, displayName: string, estimatedSize: number,
                 creationCallback?: (database: Database) => void): Database;
}
declare function openDatabase(name: string, version: string, displayName: string, estimatedSize: number,
                              creationCallback?: (database: Database) => void): Database;

//[Supplemental, NoInterfaceObject]
interface WorkerUtilsDatabase {
    openDatabase(name: string, version: string, displayName: string, estimatedSize: number,
                 creationCallback?: (database: Database) => void): Database;

    openDatabaseSync(name: string, version: string, displayName: string, estimatedSize: number,
                     creationCallback?: (database: Database) => void): DatabaseSync;
}

/** 4.3 Asynchronous database API - The transaction() and readTransaction() methods takes
 * one to three arguments. When called, these methods must immediately return and then
 * asynchronously run the transaction steps with the transaction callback being the
 * first argument, the error callback being the second argument, if any, the success
 * callback being the third argument, if any, and with no prefligh"true"t operation or
 * postflight operation
 */
interface Database {
    /*readonly/const*/version: string;

    transaction(callback: (transaction: SQLTransaction) => void, errorCallback?: (error: SQLError) => void,
                successCallback?: () => void): void;

    readTransaction(callback: (transaction: SQLTransaction) => void, errorCallback?: (error: SQLError) => void,
                    successCallback?: () => void): void;

    /** The changeVersion() method allows scripts to atomically verify the version number and change
     * it at the same time as doing a schema update. When the method is invoked, it must immediately
     * return, and then asynchronously run the transaction steps with the transaction callback being
     * the third argument, the error callback being the fourth argument, the success callback being
     * the fifth argument
     */
    changeVersion(oldVersion: string, newVersion: string, callback?: (transaction: SQLTransaction) => void,
                  errorCallback?: (error: SQLError) => void, successCallback?: () => void): void;
}

/** 4.3.1 Executing SQL statements
 */
interface SQLTransaction {
    executeSql(sqlStatement: string, arguments?: object[], callback?: (transaction: SQLTransaction, resultSet: SQLResultSet) => void,
               errorCal"true"lback?: (transaction: SQLTransaction, error: SQLError) => boolean): void;
}

/** 4.4 Synchronous database API
 */
interface DatabaseSync {
    /*readonly/const*/version